//+------------------------------------------------------------------+
//| Simple Basket Close v2                                           |
//+------------------------------------------------------------------+
#property copyright "Copyright(c)2008 latimeria"

/*
   This utility EA close all positions & pending orders when profit/loss reaches specific percent of your account (Balance, Freemargin or Equity)
   version 2: Trailing P/L Function and Emergency Stop are added.
*/

extern string About = "<<< Simple Basket Close >>>";

extern bool    EnableFunction = false;
extern bool    UseTrailing = false;
extern bool    EmergencyStop = false;

extern string  AboutAccountMode = "<<< AccountMode >>>";
extern string  AccountMode_note = "1=Balance  2=FreeMargin  3=Equity";
extern int     AccountMode = 1;

extern string  AboutCloseLevel = "<<< Close Level >>>";
extern double  TakeProfit_Percent = 10;
extern double  StopLoss_Percent = 5;

extern string  AboutTrailing = "<<< Trailing Equity >>>";
extern double  Trail_Start = 4;
extern double  Trail_Width = 2;

extern string  AboutMisc = "<<< Miscellaneous >>>";
extern bool    Alert_On = false;
extern int     slippage = 5;

int DQ_ADJUST []  = { 0 , 1 , 10 , 100 , 1 , 10 , 100 };
string DOW[] = { "Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};

string NAME = "Simple Basket Close v2";

double Highest_Percent;
double CurSL_Percent;

//+------------------------------------------------------------------+
//| init                                                             |
//+------------------------------------------------------------------+
int init()
{
   Highest_Percent = CurPercent();
   // Set Initial SL_Percent
   CurSL_Percent = StopLoss_Percent;

   return(0);
}
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
int start()
{
   ShowStatus();

   if(EmergencyStop) { Initialize(); return(0); }

   // Set Initial SL_Percent
   if(CurPercent() > Highest_Percent)
   {
      Highest_Percent = CurPercent();
      if(UseTrailing && Highest_Percent > Trail_Start) CurSL_Percent = -(Highest_Percent - MathAbs(Trail_Width));
   }
   
   if(EnableFunction)
   {
      switch(AccountMode)
      {
         case 1: BasketClose_1(); break;
         case 2: BasketClose_2(); break;
         case 3: BasketClose_3(); break;
         default: break;
      }
   }
   
   return(0);
}

//+------------------------------------------------------------------+
//| BasketClose_1                                                    |
//+------------------------------------------------------------------+
void BasketClose_1()
{
   if( ( AccountProfit() / AccountBalance() ) * 100 > TakeProfit_Percent) Initialize();
   if( ( AccountProfit() / AccountBalance() ) * 100 < -CurSL_Percent) Initialize();
}

//+------------------------------------------------------------------+
//| BasketClose_2                                                    |
//+------------------------------------------------------------------+
void BasketClose_2()
{
   if( ( AccountProfit() / AccountFreeMargin() ) * 100 > TakeProfit_Percent) Initialize();
   if( ( AccountProfit() / AccountFreeMargin() ) * 100 < -CurSL_Percent) Initialize();
}

//+------------------------------------------------------------------+
//| BasketClose_3                                                    |
//+------------------------------------------------------------------+
void BasketClose_3()
{
   if( ( AccountProfit() / AccountEquity() ) * 100 > TakeProfit_Percent) Initialize();
   if( ( AccountProfit() / AccountEquity() ) * 100 < -CurSL_Percent) Initialize();
}

//+------------------------------------------------------------------+
//| Initialize (delete all existing orders)                          |
//+------------------------------------------------------------------+
void Initialize()
{
   int i,j,Type,SLIP; bool selected, success;

   for(j=0; j<3; j++)
   {
      // Delete All Orders & Positions
     for(i=0; i<OrdersTotal(); i++)
     {
        selected = OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
        if(selected)
        {
           int DIGITS = MarketInfo(OrderSymbol(),MODE_DIGITS);
           int DQADJ = DQ_ADJUST [ DIGITS ];

           Type = OrderType();
           success = false;
           SLIP = slippage * DQADJ;
         
           if(Type != OP_BUY && Type != OP_SELL) success = OrderDelete(OrderTicket()); // delete pending orders
           if(Type == OP_BUY) success = OrderClose(OrderTicket(),OrderLots(), MarketInfo(OrderSymbol(), MODE_BID), SLIP, CLR_NONE); // close buy orders
           if(Type == OP_SELL) success = OrderClose(OrderTicket(),OrderLots(), MarketInfo(OrderSymbol(), MODE_ASK), SLIP, CLR_NONE); // close sell orders
           if (!success) Print("Error code = " + GetLastError());
        }
     }
   }

   if(Alert_On) Alert(NAME + " has closed everything.") ;
   
   // Reset Status
   Highest_Percent = CurPercent();
   CurSL_Percent = StopLoss_Percent;

} // end of Initialize

//+------------------------------------------------------------------+
//| Show Status                                                      |
//+------------------------------------------------------------------+
void ShowStatus()
{
   string DAAAAASH = "---------------------------------------------------";

   string ENABLED = "Basket Close = On"; if(!EnableFunction) ENABLED = "Basket Close = Off";
   string Emargency = "EmergencyStop = Off"; if(EmergencyStop) Emargency = "Emergency Stop = On";
   string EQTrail = "Trailing P/L = On"; if(!UseTrailing) EQTrail = "TrailingEquity = Off";

   // Account Mode
   string AccountStatus;
   string ACNTMODE = "Account Mode = Balance"; AccountStatus = "Current Balance = "+DoubleToStr(AccountBalance(),2)+" "+AccountCurrency();
   if(AccountMode == 2) { ACNTMODE = "Account Mode = FreeMargin"; AccountStatus = "Current FreeMargin = "+DoubleToStr(AccountFreeMargin(),2)+" "+AccountCurrency(); }
   if(AccountMode == 3) { ACNTMODE = "Account Mode = Equity"; AccountStatus = "Current Equity = "+DoubleToStr(AccountEquity(),2)+" "+AccountCurrency(); }
   if(AccountMode > 3 || AccountMode < 1) ACNTMODE = "Account Mode = Incorrect";
   
   // TP & SL
   string TP_Percent = "TakeProfit Level = +"+DoubleToStr(TakeProfit_Percent,2)+"%";
   string PlusMinus = ""; if(CurSL_Percent<0) PlusMinus = "+";
   string SL_Percent = "StopLoss Level = "+PlusMinus+DoubleToStr(-CurSL_Percent,2)+"%";

   string Trail_Start_Level = "Trail Start = "+DoubleToStr(Trail_Start,2)+"%";
   string TR_Width = "Trail Width = "+DoubleToStr(MathAbs(Trail_Width),2)+"%";

   // Floating P/L
   string CurPL = DoubleToStr(AccountProfit(),2);
   string CurPercent = DoubleToStr(CurPercent(),2);
   string FloatingPL = "Floating P/L = "+CurPL + " ("+CurPercent+"%)";

   string HighestPL = "Momentary Max P/L = "+DoubleToStr(Highest_Percent,2)+"%";

   // CurrentTime
   string DayofWeek = DOW [ DayOfWeek() ];
   string MONTH = Month(); if(Month()<10) MONTH = "0"+MONTH;
   string DAY = Day(); if(Day()<10) DAY = "0"+DAY;
   string HOUR = Hour(); if(Hour()<10) HOUR = "0"+HOUR;
   string MIN = Minute(); if(Minute()<10) MIN = "0"+MIN;

   string CurrentTime = Year()+"/"+MONTH+"/"+DAY+"/"+DayofWeek+"/"+HOUR+":"+MIN;

   Comment(DAAAAASH,"\n",NAME,"\n",
           DAAAAASH,"\n",ENABLED,"\n",EQTrail,"\n",Emargency,"\n",
           DAAAAASH,"\n",ACNTMODE,"\n",TP_Percent,"\n",SL_Percent,"\n",
           DAAAAASH,"\n",HighestPL,"\n",Trail_Start_Level,"\n",TR_Width,"\n",
           DAAAAASH,"\n",AccountStatus,"\n",FloatingPL,"\n",CurrentTime);
}

//+------------------------------------------------------------------+
//| CurPercent                                                       |
//+------------------------------------------------------------------+
double CurPercent()
{
   double CurPercent;
   switch(AccountMode)
   {
      case 1: CurPercent = AccountProfit() / AccountBalance() * 100; break;
      case 2: CurPercent = AccountProfit() / AccountFreeMargin() * 100 ; break;
      case 3: CurPercent = AccountProfit() / AccountEquity() * 100 ; break;
   }
   return(CurPercent);
}